; =====================================================================
; PPM-Summensignalbildung fr 2,4GHz-Empfnger R617FS mit ATTiny2313
; =====================================================================
;
; Liest Servoimpulse Kanal 1-7 vom Empfnger ein und fasst sie zum
; Summensignal fr die Fl-Ctrl des Mikrokopters zusammen.
;
; Auerdem wird die LED des R617FS gelesen und bei fehlendem Empfang
; kein PPM-Signal ausgegeben. Damit kann die Fl-Ctrl den Failsafe-
; Zustand erkennen und das Notlandeprogramm starten.
;
; =====================================================================
; Erstellt von Jrg Reisener am:			23.01.2009
; letzte Bearbeitung:					04.02.2009
; =====================================================================
;
; Einstellung der Konfigurationsbits (Fuse-Bits) des Tiny2313:
; (interner 8MHz-Takt ohne Vorteiler)
;
; CKDIV8			 = 1
; CKOUT				 = 1
; SUT1, SUT0			 = 10
; CKSEL3, CKSEL2, CKSEL1, CKSEL0 = 0100
; alle anderen			 = 1 (unchecked)
; =====================================================================
;
; ----------------------
; Belegung der I/O-Ports
; ----------------------
;
; PortB0: nicht genutzt
; PortB1: AIN1 Eingang Fotowiderstand
; PortB2: Eingang Kanal 1
; PortB3: Eingang Kanal 3
; PortB4: Eingang Kanal 5
; PortB5: (MOSI)
; PortB6: (MISO)
; PortB7: (SCK)
;
; PortD0: Ausgang PPM-Signal
; PortD1: nicht genutzt
; PortD2: INT0 Eingang Kanal 7
; PortD3: INT1 Eingang Kanal 6
; PortD4: nicht genutzt
; PortD5: nicht genutzt
; PortD6: nicht genutzt
;
; ------------------
; Belegung der Timer
; ------------------
;
; Timer0: nicht genutzt
; Timer1: Impulslnge Kanal 6-7
;
; -----------------------------------
; Konstanten und Registerdefinitionen
; -----------------------------------
;
.nolist
.include "tn2313def.inc"
.list
;
.equ	xtal=	8000000	;Controller Taktfrequenz 8 MHz, intern
;
.def	insreg=	r1	;Zwischenspeicher fr SREG whrend Interrupt
.def	k6anfl=	r3	;Kanal 6 Startzeit low
.def	k6anfh=	r4	;Kanal 6 Startzeit high
.def	k7anfl=	r5	;Kanal 7 Startzeit low
.def	k7anfh=	r6	;Kanal 7 Startzeit high
.def	k6endl=	r20	;Kanal 6 Endzeit low
.def	k6endh=	r21	;Kanal 6 Endzeit high
.def	k7endl=	r22	;Kanal 7 Endzeit low
.def	k7endh=	r23	;Kanal 7 Endzeit high
.def	flags=	r24	;verschiedene Flags
			;Bit0: wird im Timer Compare Interrupt gesetzt
			;      und im Hauptprogramm gelscht
.def	itemp=	r25	;Zwischenspeicher whrend Interruptbearbeitung
; --------------
; SRAM Variablen
; --------------
;
.dseg
.org	0x60
;
; ---------------
; Programmbereich
; ---------------
;
.cseg
.org	0
;
; Interrupt-Vektoren
;
reset:	rjmp	start			;Programmstart
	rjmp	exint0			;ext. Interrupt Kanal 7
	rjmp	exint1			;ext. Interrupt Kanal 6
	reti
	rjmp	t1comp			;Timer1 Compare Match A
;
.db	"PPM-Summensignal    "
.db	"fuer futaba R617FS  "
.db	"01/2009             "
.db	"(c) Joerg Reisener  "
;
; Initialisierung SRAM, I/O-Ports, Hardware
;
start:	ldi	zl,0x60
	ldi	zh,0			;Anfangsadresse SRAM
	clr	r16
sramcl:	st	z+,r16			;Speicherzelle lschen
	cpi	zl,ramend+1		;letzte Speicherzelle erreicht?
	brne	sramcl			;nein -> Schleife
	ldi	r16,ramend
	out	spl,r16			;Stackpointer setzen
	clr	r16
;
	ldi	r16,0b00000000		;PortB, B7-B0 Ein
	out	ddrb,r16
	ldi	r16,0b11111111		;PortB, B7-B0 Pullup
	out	portb,r16
	ldi	r16,0b00000001		;PortD, D0 Aus, D6-D1 Ein
	out	ddrd,r16
	ldi	r16,0b01111111		;PortD, D6-D0 Pullup bzw. high
	out	portd,r16
;
	ldi	r16,(1<<cs10)		;Timer1 kein Vorteiler
	out	tccr1b,r16
;
	ldi	r16,(1<<isc10)|(1<<isc00) ;INT0, INT1 beide Flanken
	out	mcucr,r16
	ldi	r16,(1<<int1)|(1<<int0)	;erlaube Interrupts
	out	gimsk,r16
;
	ldi	r16,(1<<acbg)		;AnalogComparator Referenz ein
	out	acsr,r16
;
	ldi	r16,(1<<ain1d)		;AIN1 Digital Input deakt.
	out	didr,r16
;
	clr	flags			;Flags lschen
	rcall	w25ms			;50ms warten
	rcall	w25ms
	sei				;Interrupt freigeben
;
; Hauptprogramm
;
;
; Fehler-LED des Empfngers prfen
;
main:	in	r16,acsr		;Analog Comparator lesen
	bst	r16,aco			;Wert < 1,1V ?
	brts	main			;ja -> Schleife
;
; Kanle 1-5 einlesen und bearbeiten
;
ma0010:	in	r16,pinb		;Servosignale einlesen
	bst	r16,2			;Kanal1 =1 ?
	brtc	ma0010			;nein -> warten
	rcall	impuls			;Impuls ausgeben
;
ma0020:	in	r16,pinb		;Servosignale einlesen
	bst	r16,2			;Kanal1 =0 ?
	brts	ma0020			;nein -> warten
	rcall	impuls			;Impuls ausgeben
;
ma0030:	in	r16,pinb		;Servosignale einlesen
	bst	r16,3			;Kanal3 =1 ?
	brtc	ma0030			;nein -> warten
	rcall	impuls			;Impuls ausgeben
;
ma0040:	in	r16,pinb		;Servosignale einlesen
	bst	r16,3			;Kanal3 =0 ?
	brts	ma0040			;nein -> warten
	rcall	impuls			;Impuls ausgeben
;
ma0050:	in	r16,pinb		;Servosignale einlesen
	bst	r16,4			;Kanal5 =1 ?
	brtc	ma0050			;nein -> warten
	rcall	impuls			;Impuls ausgeben
;
ma0060:	in	r16,pinb		;Servosignale einlesen
	bst	r16,4			;Kanal5 =0 ?
	brts	ma0060			;nein -> warten
	in	r16,pind
	bst	r16,3			;K6 angeschlossen?
	brts	ma0100			;nein -> berspringen
;
; Timer umstellen auf Compare Match
;
	clr	r16
	out	tcnt1h,r16		;Zhler auf null
	out	tcnt1l,r16
	ldi	r16,(1<<wgm12)|(1<<cs10) ;CTC-Mode, kein Vorteiler
	out	tccr1b,r16
	ldi	r16,(1<<ocie1a)		;erlaube Compare Match Int.
	out	timsk,r16
	sub	k6endl,k6anfl		;ermittle K6 Lnge
	sbc	k6endh,k6anfh
	out	ocr1ah,k6endh		;Compare Match laden
	out	ocr1al,k6endl
	rcall	impuls			;Impuls ausgeben
	cbr	flags,1			;Flag lschen
;
; Warten auf Compare Match Interrupt (Ende K6)
;
ma0070:	bst	flags,0			;Flag gesetzt?
	brtc	ma0070			;nein -> Schleife
	sub	k7endl,k7anfl		;ermittle K7 Lnge
	sbc	k7endh,k7anfh
	out	ocr1ah,k7endh		;Compare Match laden
	out	ocr1al,k7endl
	rcall	impuls			;Impuls ausgeben
	cbr	flags,1			;Flag lschen
;
; Warten auf Compare Match Interrupt (Ende K7)
;
ma0080:	bst	flags,0			;Flag gesetzt?
	brtc	ma0080			;nein -> Schleife
	rcall	impuls			;Impuls ausgeben
;
; Timer zurckstellen auf Normal Mode
;
	ldi	r16,(1<<cs10)		;Normal-Mode, kein Vorteiler
	out	tccr1b,r16
	clr	r16			;sperre Compare Match Int.
	out	timsk,r16
	cbr	flags,1			;Flag lschen
	rjmp	main
;
ma0100:	rcall	impuls			;Impuls ausgeben
	rjmp	main
;
; ----------------------
; Unterprogramm-Routinen
; ----------------------
;
; PPM-Impuls ausgeben
;
impuls:	cbi	portd,0			;negativer Impuls
	rcall	w300ys			;300ys warten
	sbi	portd,0			;Impuls-Ende
	ret
;
; Warteschleife 300s (berechnet 291ys)
;
w300ys:	push	r16
	push	r17
	clr	r16
	ldi	r17,3			;291ys
wloop:	dec	r16			;innere Schleife
	brne	wloop			;luft 96ys
	dec	r17
	brne	wloop			;uere Schleife
	pop	r17
	pop	r16
	ret
;
; Warteschleife 25ms
;
w25ms:	push	r16
	push	r17
	clr	r16
	clr	r17
	rjmp	wloop
;
; ------------------
; Interrupt-Routinen
; ------------------
;
; Externer Interrupt Kanal 7
;
exint0:	in	k7endl,tcnt1l		;Timer auslesen
	in	k7endh,tcnt1h
	in	insreg,sreg		;SREG sichern
	in	itemp,pind		;PortD einlesen
	bst	itemp,2			;Kanal 7 high?
	brtc	ex0090			;nein -> berspringen
	mov	k7anfl,k7endl		;Anfangswerte speichern
	mov	k7anfh,k7endh
ex0090:	out	sreg,insreg		;SREG wiederherstellen
	reti
;
; Externer Interrupt Kanal 6
;
exint1:	in	k6endl,tcnt1l		;Timer auslesen
	in	k6endh,tcnt1h
	in	insreg,sreg		;SREG sichern
	in	itemp,pind		;PortD einlesen
	bst	itemp,3			;Kanal 6 high?
	brtc	ex0090			;nein -> berspringen
	mov	k6anfl,k6endl		;Anfangswerte speichern
	mov	k6anfh,k6endh
ex1090:	out	sreg,insreg		;SREG wiederherstellen
	reti
;
; Timer1 Compare Match A Interrupt
;
t1comp:	in	insreg,sreg		;SREG sichern
	sbr	flags,1			;Flag setzen
	out	sreg,insreg		;SREG wiederherstellen
	reti
;
